@@ -94,6 +94,8 @@ trait MultipleReadersSingleWriterCache[W, Winfo] {
   /** Subclasses: Toggle this to enable/disable caching for your entity type. */
   protected val cacheEnabled = true
   protected val evictionPolicy: EvictionPolicy = AccessTime
+  protected val cacheExpirationTime: Long = 5
+  protected val cacheExpirationTimeUnit: TimeUnit = TimeUnit.MINUTES
   protected val fixedCacheSize = 0
 
   private object Entry {
@@ -376,7 +378,7 @@ trait MultipleReadersSingleWriterCache[W, Winfo] {
       case Failure(t) =>
         // oops, the datastore read failed. invalidate the cache entry
         // note: that this might be a perfectly legitimate failure,
-        // e.g. a lookup for a non-existant key; we need to pass the particular t through
+        // e.g. a lookup for a non-existent key; we need to pass the particular t through
         invalidateEntry(key, entry)
         promise.failure(t)
     }
@@ -452,8 +454,8 @@ trait MultipleReadersSingleWriterCache[W, Winfo] {
       .softValues()
 
     evictionPolicy match {
-      case AccessTime => b.expireAfterAccess(5, TimeUnit.MINUTES)
-      case _          => b.expireAfterWrite(5, TimeUnit.MINUTES)
+      case AccessTime => b.expireAfterAccess(cacheExpirationTime, cacheExpirationTimeUnit)
+      case _          => b.expireAfterWrite(cacheExpirationTime, cacheExpirationTimeUnit)
     }
 
     if (fixedCacheSize > 0) b.maximumSize(fixedCacheSize)
