@@ -18,7 +18,6 @@
 package org.apache.openwhisk.core.controller
 
 import java.time.{Clock, Instant}
-
 import scala.collection.immutable.Map
 import scala.concurrent.Future
 import scala.util.Try
@@ -28,13 +27,11 @@ import akka.http.scaladsl.marshallers.sprayjson.SprayJsonSupport._
 import akka.http.scaladsl.model.HttpMethods.POST
 import akka.http.scaladsl.model.StatusCodes.{Accepted, BadRequest, InternalServerError, NoContent, OK, ServerError}
 import akka.http.scaladsl.model.Uri.Path
-import akka.http.scaladsl.model.headers.Authorization
+import akka.http.scaladsl.model.headers.{`Timeout-Access`, Authorization}
 import akka.http.scaladsl.model._
 import akka.http.scaladsl.server.{RequestContext, RouteResult}
 import akka.http.scaladsl.unmarshalling.{Unmarshal, Unmarshaller}
-import akka.stream.ActorMaterializer
 import spray.json.DefaultJsonProtocol._
-import com.typesafe.sslconfig.akka.AkkaSSLConfig
 import pureconfig._
 import pureconfig.generic.auto._
 import spray.json._
@@ -63,12 +60,8 @@ trait WhiskTriggersApi extends WhiskCollectionAPI {
 
   /** Connection context for HTTPS */
   protected lazy val httpsConnectionContext = {
-    val sslConfig = AkkaSSLConfig().mapSettings { s =>
-      s.withLoose(s.loose.withDisableHostnameVerification(true))
-    }
     val httpsConfig = loadConfigOrThrow[HttpsConfig]("whisk.controller.https")
-    Https.connectionContext(httpsConfig, Some(sslConfig))
-
+    Https.connectionContextClient(httpsConfig, true)
   }
 
   protected val controllerProtocol = loadConfigOrThrow[String]("whisk.controller.protocol")
@@ -96,8 +89,6 @@ trait WhiskTriggersApi extends WhiskCollectionAPI {
   protected val triggersPath = "triggers"
   protected val url = Uri(s"${controllerProtocol}://localhost:${whiskConfig.servicePort}")
 
-  protected implicit val materializer: ActorMaterializer
-
   import RestApiCommons.emptyEntityToJsObject
 
   /**
@@ -159,7 +150,19 @@ trait WhiskTriggersApi extends WhiskCollectionAPI {
                 response = ActivationResponse.success(payload orElse Some(JsObject.empty)),
                 version = trigger.version,
                 duration = None)
-              val args: JsObject = trigger.parameters.merge(payload).getOrElse(JsObject.empty)
+              val headers = JsObject(
+                Map(new WebApiDirectives().headers -> request.headers
+                  .collect {
+                    case h if h.name != `Timeout-Access`.name => h.lowercaseName -> h.value
+                  }
+                  .toMap
+                  .toJson))
+
+              val mergedPayload = Some {
+                (headers.fields ++ (payload getOrElse JsObject.empty).fields).toJson.asJsObject
+              }
+
+              val args: JsObject = trigger.parameters.merge(mergedPayload).getOrElse(JsObject.empty)
 
               activateRules(user, args, trigger.rules.getOrElse(Map.empty))
                 .map(results => triggerActivation.withLogs(ActivationLogs(results.map(_.toJson.compactPrint).toVector)))
@@ -169,7 +172,7 @@ trait WhiskTriggersApi extends WhiskCollectionAPI {
                     triggerActivation
                 }
                 .map { activation =>
-                  activationStore.storeAfterCheck(activation, false, None, context)
+                  activationStore.storeAfterCheck(activation, false, None, None, context)
                 }
 
               respondWithActivationIdHeader(triggerActivationId) {
