@@ -29,6 +29,7 @@ import akka.http.scaladsl.marshallers.sprayjson.SprayJsonSupport._
 import akka.http.scaladsl.model.FormData
 import akka.http.scaladsl.model.HttpEntity
 import akka.http.scaladsl.model.MediaTypes
+import akka.http.scaladsl.model.StatusCodes
 import akka.http.scaladsl.model.StatusCodes._
 import akka.http.scaladsl.model.HttpCharsets
 import akka.http.scaladsl.model.HttpHeader
@@ -117,12 +118,21 @@ class WebActionsApiTests extends FlatSpec with Matchers with WebActionsApiBaseTe
 }
 
 trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEach with WhiskWebActionsApi {
+
+  val systemPayloadLimit = ActivationEntityLimit.MAX_ACTIVATION_ENTITY_LIMIT
+  val namespacePayloadLimit = systemPayloadLimit - 100.KB
+
   val uuid = UUID()
   val systemId = Subject()
   val systemKey = BasicAuthenticationAuthKey(uuid, Secret())
   val systemIdentity =
     Future.successful(
-      Identity(systemId, Namespace(EntityName(systemId.asString), uuid), systemKey, rights = Privilege.ALL))
+      Identity(
+        systemId,
+        Namespace(EntityName(systemId.asString), uuid),
+        systemKey,
+        rights = Privilege.ALL,
+        limits = UserLimits(maxPayloadSize = Option(namespacePayloadLimit))))
   val namespace = EntityPath(systemId.asString)
   val proxyNamespace = namespace.addPath(EntityName("proxy"))
   override lazy val entitlementProvider = new TestingEntitlementProvider(whiskConfig, loadBalancer)
@@ -139,6 +149,7 @@ trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEac
   var requireAuthenticationKey = "example-web-action-api-key"
   var invocationCount = 0
   var invocationsAllowed = 0
+
   lazy val testFixturesToGc = {
     implicit val tid = transid()
     Seq(
@@ -260,7 +271,7 @@ trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEac
   override protected[controller] def invokeAction(
     user: Identity,
     action: WhiskActionMetaData,
-    payload: Option[JsObject],
+    payload: Option[JsValue],
     waitForResponse: Option[FiniteDuration],
     cause: Option[ActivationId])(implicit transid: TransactionId): Future[Either[ActivationId, WhiskActivation]] = {
     invocationCount = invocationCount + 1
@@ -270,10 +281,14 @@ trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEac
       // 1. the package name for the action (to confirm that this resolved to systemId)
       // 2. the action name (to confirm that this resolved to the expected action)
       // 3. the payload received by the action which consists of the action.params + payload
+      val content = payload match {
+        case Some(JsObject(fields)) => action.parameters.merge(Some(JsObject(fields)))
+        case _                      => Some(action.parameters.toJsObject)
+      }
       val result = actionResult getOrElse JsObject(
         "pkg" -> action.namespace.toJson,
         "action" -> action.name.toJson,
-        "content" -> action.parameters.merge(payload).get)
+        "content" -> content.get)
 
       val activation = WhiskActivation(
         action.namespace,
@@ -772,18 +787,18 @@ trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEac
       put(entityStore, reference)
       put(entityStore, action)
 
-      val contentX = JsObject("x" -> "overriden".toJson)
-      val contentZ = JsObject("z" -> "overriden".toJson)
+      val contentX = JsObject("x" -> "overridden".toJson)
+      val contentZ = JsObject("z" -> "overridden".toJson)
 
       allowedMethodsWithEntity.foreach { m =>
         invocationsAllowed += 1
 
-        m(s"$testRoutePath/$systemId/${reference.name}/export_c.json?x=overriden") ~> Route.seal(routes(creds)) ~> check {
+        m(s"$testRoutePath/$systemId/${reference.name}/export_c.json?x=overridden") ~> Route.seal(routes(creds)) ~> check {
           status should be(BadRequest)
           responseAs[ErrorResponse].error shouldBe Messages.parametersNotAllowed
         }
 
-        m(s"$testRoutePath/$systemId/${reference.name}/export_c.json?y=overriden") ~> Route.seal(routes(creds)) ~> check {
+        m(s"$testRoutePath/$systemId/${reference.name}/export_c.json?y=overridden") ~> Route.seal(routes(creds)) ~> check {
           status should be(BadRequest)
           responseAs[ErrorResponse].error shouldBe Messages.parametersNotAllowed
         }
@@ -793,13 +808,13 @@ trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEac
           responseAs[ErrorResponse].error shouldBe Messages.parametersNotAllowed
         }
 
-        m(s"$testRoutePath/$systemId/${reference.name}/export_c.json?y=overriden", contentZ) ~> Route.seal(
+        m(s"$testRoutePath/$systemId/${reference.name}/export_c.json?y=overridden", contentZ) ~> Route.seal(
           routes(creds)) ~> check {
           status should be(BadRequest)
           responseAs[ErrorResponse].error shouldBe Messages.parametersNotAllowed
         }
 
-        m(s"$testRoutePath/$systemId/${reference.name}/export_c.json?empty=overriden") ~> Route.seal(routes(creds)) ~> check {
+        m(s"$testRoutePath/$systemId/${reference.name}/export_c.json?empty=overridden") ~> Route.seal(routes(creds)) ~> check {
           status should be(OK)
           val response = responseAs[JsObject]
           response shouldBe JsObject(
@@ -807,7 +822,7 @@ trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEac
             "action" -> "export_c".toJson,
             "content" -> metaPayload(
               m.method.name.toLowerCase,
-              Map("empty" -> "overriden").toJson.asJsObject,
+              Map("empty" -> "overridden").toJson.asJsObject,
               creds,
               pkgName = "proxy"))
         }
@@ -914,6 +929,21 @@ trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEac
       }
     }
 
+    it should s"use non-standard action status code to terminate an http response (auth? ${creds.isDefined})" in {
+      implicit val tid = transid()
+
+      Seq(s"$systemId/proxy/export_c.http").foreach { path =>
+        allowedMethods.foreach { m =>
+          actionResult = Some(JsObject(webApiDirectives.statusCode -> JsNumber(444)))
+          invocationsAllowed += 1
+
+          m(s"$testRoutePath/$path") ~> Route.seal(routes(creds)) ~> check {
+            status should be(StatusCodes.custom(444, ""))
+          }
+        }
+      }
+    }
+
     it should s"use default field projection for extension (auth? ${creds.isDefined})" in {
       implicit val tid = transid()
 
@@ -1484,17 +1514,18 @@ trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEac
       }
     }
 
-    it should s"reject requests when entity size exceeds allowed limit (auth? ${creds.isDefined})" in {
+    it should s"reject requests when entity size exceeds allowed system limit (auth? ${creds.isDefined})" in {
       implicit val tid = transid()
 
       Seq(s"$systemId/proxy/export_c.json").foreach { path =>
-        val largeEntity = "a" * (allowedActivationEntitySize.toInt + 1)
+        val largeEntity = "a" * (systemPayloadLimit.toBytes.toInt + 1)
 
         val content = s"""{"a":"$largeEntity"}"""
         Post(s"$testRoutePath/$path", content.parseJson.asJsObject) ~> Route.seal(routes(creds)) ~> check {
           status should be(PayloadTooLarge)
           val expectedErrorMsg = Messages.entityTooBig(
-            SizeError(fieldDescriptionForSizeError, (largeEntity.length + 8).B, allowedActivationEntitySize.B))
+            // must contains namespace's payload limit size in error message
+            SizeError(fieldDescriptionForSizeError, (largeEntity.length + 8).B, namespacePayloadLimit.toBytes.B))
           confirmErrorWithTid(responseAs[JsObject], Some(expectedErrorMsg))
         }
 
@@ -1502,7 +1533,50 @@ trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEac
         Post(s"$testRoutePath/$path", form) ~> Route.seal(routes(creds)) ~> check {
           status should be(PayloadTooLarge)
           val expectedErrorMsg = Messages.entityTooBig(
-            SizeError(fieldDescriptionForSizeError, (largeEntity.length + 2).B, allowedActivationEntitySize.B))
+            // must contains namespace's payload limit size in error message
+            SizeError(fieldDescriptionForSizeError, (largeEntity.length + 2).B, namespacePayloadLimit.toBytes.B))
+          confirmErrorWithTid(responseAs[JsObject], Some(expectedErrorMsg))
+        }
+      }
+    }
+
+    it should s"allow requests when entity size does not exceed allowed namespace limit (auth? ${creds.isDefined})" in {
+      implicit val tid = transid()
+
+      Seq(s"$systemId/proxy/export_c.json").foreach { path =>
+        val largeEntity = "a" * (namespacePayloadLimit.toBytes.toInt - 10)
+
+        val content = s"""{"a":"$largeEntity"}"""
+        Post(s"$testRoutePath/$path", content.parseJson.asJsObject) ~> Route.seal(routes(creds)) ~> check {
+          status should be(OK)
+        }
+
+        val form = FormData(Map("a" -> largeEntity))
+        Post(s"$testRoutePath/$path", form) ~> Route.seal(routes(creds)) ~> check {
+          status should be(OK)
+        }
+      }
+    }
+
+    it should s"reject requests when entity size exceeds allowed namespace limit (auth? ${creds.isDefined})" in {
+      implicit val tid = transid()
+
+      Seq(s"$systemId/proxy/export_c.json").foreach { path =>
+        val largeEntity = "a" * (namespacePayloadLimit.toBytes.toInt + 1)
+
+        val content = s"""{"a":"$largeEntity"}"""
+        Post(s"$testRoutePath/$path", content.parseJson.asJsObject) ~> Route.seal(routes(creds)) ~> check {
+          status should be(PayloadTooLarge)
+          val expectedErrorMsg = Messages.entityTooBig(
+            SizeError(fieldDescriptionForSizeError, (largeEntity.length + 8).B, namespacePayloadLimit.toBytes.B))
+          confirmErrorWithTid(responseAs[JsObject], Some(expectedErrorMsg))
+        }
+
+        val form = FormData(Map("a" -> largeEntity))
+        Post(s"$testRoutePath/$path", form) ~> Route.seal(routes(creds)) ~> check {
+          status should be(PayloadTooLarge)
+          val expectedErrorMsg = Messages.entityTooBig(
+            SizeError(fieldDescriptionForSizeError, (largeEntity.length + 2).B, namespacePayloadLimit.toBytes.B))
           confirmErrorWithTid(responseAs[JsObject], Some(expectedErrorMsg))
         }
       }
@@ -1554,18 +1628,18 @@ trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEac
 
     it should s"reject request that tries to override final parameters (auth? ${creds.isDefined})" in {
       implicit val tid = transid()
-      val contentX = JsObject("x" -> "overriden".toJson)
-      val contentZ = JsObject("z" -> "overriden".toJson)
+      val contentX = JsObject("x" -> "overridden".toJson)
+      val contentZ = JsObject("z" -> "overridden".toJson)
 
       allowedMethodsWithEntity.foreach { m =>
         invocationsAllowed += 1
 
-        m(s"$testRoutePath/$systemId/proxy/export_c.json?x=overriden") ~> Route.seal(routes(creds)) ~> check {
+        m(s"$testRoutePath/$systemId/proxy/export_c.json?x=overridden") ~> Route.seal(routes(creds)) ~> check {
           status should be(BadRequest)
           responseAs[ErrorResponse].error shouldBe Messages.parametersNotAllowed
         }
 
-        m(s"$testRoutePath/$systemId/proxy/export_c.json?y=overriden") ~> Route.seal(routes(creds)) ~> check {
+        m(s"$testRoutePath/$systemId/proxy/export_c.json?y=overridden") ~> Route.seal(routes(creds)) ~> check {
           status should be(BadRequest)
           responseAs[ErrorResponse].error shouldBe Messages.parametersNotAllowed
         }
@@ -1575,12 +1649,12 @@ trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEac
           responseAs[ErrorResponse].error shouldBe Messages.parametersNotAllowed
         }
 
-        m(s"$testRoutePath/$systemId/proxy/export_c.json?y=overriden", contentZ) ~> Route.seal(routes(creds)) ~> check {
+        m(s"$testRoutePath/$systemId/proxy/export_c.json?y=overridden", contentZ) ~> Route.seal(routes(creds)) ~> check {
           status should be(BadRequest)
           responseAs[ErrorResponse].error shouldBe Messages.parametersNotAllowed
         }
 
-        m(s"$testRoutePath/$systemId/proxy/export_c.json?empty=overriden") ~> Route.seal(routes(creds)) ~> check {
+        m(s"$testRoutePath/$systemId/proxy/export_c.json?empty=overridden") ~> Route.seal(routes(creds)) ~> check {
           status should be(OK)
           val response = responseAs[JsObject]
           response shouldBe JsObject(
@@ -1588,7 +1662,7 @@ trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEac
             "action" -> "export_c".toJson,
             "content" -> metaPayload(
               m.method.name.toLowerCase,
-              Map("empty" -> "overriden").toJson.asJsObject,
+              Map("empty" -> "overridden").toJson.asJsObject,
               creds,
               pkgName = "proxy"))
         }
@@ -1783,7 +1857,7 @@ trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEac
       implicit val tid = transid()
       invocationsAllowed = 2
 
-      val queryString = "x=overriden&key2=value2"
+      val queryString = "x=overridden&key2=value2"
       Post(s"$testRoutePath/$systemId/proxy/raw_export_c.json?$queryString") ~> Route.seal(routes(creds)) ~> check {
         status should be(OK)
         val response = responseAs[JsObject]
@@ -1799,7 +1873,7 @@ trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEac
 
       Post(
         s"$testRoutePath/$systemId/proxy/raw_export_c.json",
-        JsObject("x" -> "overriden".toJson, "key2" -> "value2".toJson)) ~> Route.seal(routes(creds)) ~> check {
+        JsObject("x" -> "overridden".toJson, "key2" -> "value2".toJson)) ~> Route.seal(routes(creds)) ~> check {
         status should be(OK)
         val response = responseAs[JsObject]
         response shouldBe JsObject(
@@ -1808,7 +1882,7 @@ trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEac
           "content" -> metaPayload(
             Post.method.name.toLowerCase,
             Map(webApiDirectives.query -> "".toJson, webApiDirectives.body -> Base64.getEncoder.encodeToString {
-              JsObject("x" -> JsString("overriden"), "key2" -> JsString("value2")).compactPrint.getBytes
+              JsObject("x" -> JsString("overridden"), "key2" -> JsString("value2")).compactPrint.getBytes
             }.toJson).toJson.asJsObject,
             creds,
             pkgName = "proxy",
@@ -1957,13 +2031,18 @@ trait WebActionsApiBaseTests extends ControllerTestCommon with BeforeAndAfterEac
 
     it should s"allowed string based status code (auth? ${creds.isDefined})" in {
       implicit val tid = transid()
-      invocationsAllowed += 2
+      invocationsAllowed += 3
 
       actionResult = Some(JsObject(webApiDirectives.statusCode -> JsString("200")))
       Head(s"$testRoutePath/$systemId/proxy/export_c.http") ~> Route.seal(routes(creds)) ~> check {
         status should be(OK)
       }
 
+      actionResult = Some(JsObject(webApiDirectives.statusCode -> JsString("444")))
+      Head(s"$testRoutePath/$systemId/proxy/export_c.http") ~> Route.seal(routes(creds)) ~> check {
+        status should be(StatusCodes.custom(444, ""))
+      }
+
       actionResult = Some(JsObject(webApiDirectives.statusCode -> JsString("xyz")))
       Head(s"$testRoutePath/$systemId/proxy/export_c.http") ~> Route.seal(routes(creds)) ~> check {
         status should be(BadRequest)
