@@ -20,13 +20,11 @@ package org.apache.openwhisk.core.loadBalancer.test
 import akka.actor.ActorRef
 import akka.actor.ActorRefFactory
 import akka.actor.ActorSystem
-import akka.stream.ActorMaterializer
 import akka.testkit.TestProbe
 import common.{StreamLogging, WhiskProperties}
-import java.nio.charset.StandardCharsets
 
-import org.apache.kafka.clients.producer.RecordMetadata
-import org.apache.kafka.common.TopicPartition
+import java.nio.charset.StandardCharsets
+import org.apache.openwhisk.common.InvokerState.{Healthy, Offline, Unhealthy}
 import org.junit.runner.RunWith
 import org.scalamock.scalatest.MockFactory
 import org.scalatest.junit.JUnitRunner
@@ -36,17 +34,18 @@ import org.scalatest.Matchers
 import scala.concurrent.Await
 import scala.concurrent.Future
 import scala.concurrent.duration._
-import org.apache.openwhisk.common.Logging
-import org.apache.openwhisk.common.NestedSemaphore
+import org.apache.openwhisk.common.{InvokerHealth, Logging, NestedSemaphore, TransactionId}
 import org.apache.openwhisk.core.entity.FullyQualifiedEntityName
-import org.apache.openwhisk.common.TransactionId
 import org.apache.openwhisk.core.WhiskConfig
-import org.apache.openwhisk.core.connector.ActivationMessage
-import org.apache.openwhisk.core.connector.CompletionMessage
-import org.apache.openwhisk.core.connector.Message
-import org.apache.openwhisk.core.connector.MessageConsumer
-import org.apache.openwhisk.core.connector.MessageProducer
-import org.apache.openwhisk.core.connector.MessagingProvider
+import org.apache.openwhisk.core.connector.{
+  ActivationMessage,
+  CompletionMessage,
+  Message,
+  MessageConsumer,
+  MessageProducer,
+  MessagingProvider,
+  ResultMetadata
+}
 import org.apache.openwhisk.core.entity.ActivationId
 import org.apache.openwhisk.core.entity.BasicAuthenticationAuthKey
 import org.apache.openwhisk.core.entity.ControllerInstanceId
@@ -67,7 +66,6 @@ import org.apache.openwhisk.core.entity.size._
 import org.apache.openwhisk.core.entity.test.ExecHelpers
 import org.apache.openwhisk.core.loadBalancer.FeedFactory
 import org.apache.openwhisk.core.loadBalancer.InvokerPoolFactory
-import org.apache.openwhisk.core.loadBalancer.InvokerState._
 import org.apache.openwhisk.core.loadBalancer._
 
 /**
@@ -127,7 +125,7 @@ class ShardingContainerPoolBalancerTests
     state.managedStepSizes shouldBe Seq(1)
     state.blackboxStepSizes shouldBe Seq(1)
 
-    // aquire a slot to alter invoker state
+    // acquire a slot to alter invoker state
     state.invokerSlots.head.tryAcquire(memoryPerSlot.toMB.toInt)
     state.invokerSlots.head.availablePermits shouldBe (memory - memoryPerSlot).toMB.toInt
 
@@ -411,7 +409,6 @@ class ShardingContainerPoolBalancerTests
 
   }
 
-  implicit val am = ActorMaterializer()
   val config = new WhiskConfig(ExecManifest.requiredProperties)
   val invokerMem = 2000.MB
   val concurrencyEnabled = Option(WhiskProperties.getProperty("whisk.action.concurrency")).exists(_.toBoolean)
@@ -421,7 +418,7 @@ class ShardingContainerPoolBalancerTests
     WhiskActionMetaData(
       namespace,
       name,
-      js10MetaData(Some("jsMain"), false),
+      jsMetaData(Some("jsMain"), false),
       limits = actionLimits(actionMem, concurrency))
   val maxContainers = invokerMem.toMB.toInt / actionMetaData.limits.memory.megabytes
   val numInvokers = 3
@@ -459,7 +456,7 @@ class ShardingContainerPoolBalancerTests
     (producer
       .send(_: String, _: Message, _: Int))
       .when(*, *, *)
-      .returns(Future.successful(new RecordMetadata(new TopicPartition("fake", 0), 0, 0, 0l, 0l, 0, 0)))
+      .returns(Future.successful(ResultMetadata("fake", 0, 0)))
 
     messaging
   }
@@ -476,7 +473,7 @@ class ShardingContainerPoolBalancerTests
         actorRefFactory: ActorRefFactory,
         messagingProvider: MessagingProvider,
         messagingProducer: MessageProducer,
-        sendActivationToInvoker: (MessageProducer, ActivationMessage, InvokerInstanceId) => Future[RecordMetadata],
+        sendActivationToInvoker: (MessageProducer, ActivationMessage, InvokerInstanceId) => Future[ResultMetadata],
         monitor: Option[ActorRef]): ActorRef =
         TestProbe().testActor
     }
