@@ -26,12 +26,13 @@ cd $ROOTDIR/tools/travis
 export TESTCONTAINERS_RYUK_DISABLED="true"
 export ORG_GRADLE_PROJECT_testSetName="REQUIRE_ONLY_DB"
 
-./scan.sh
-
 ./setupPrereq.sh
 
 cat "$ROOTDIR/tests/src/test/resources/application.conf"
 
 ./distDocker.sh
 
+# yet another hack to hit docker rate limits early...
+docker pull alpine:3.5
+
 ./runTests.sh
